/* DC9 update
 Esto es requerido por Dynamic C versin 9, para utilizar
una IP fija en programa y la interfaz Ethernet por defecto.
No es necesario en DC8, se autodefinen en las libs */
#define TCPCONFIG 0
#define USE_ETHERNET		1
/* Fin DC9 update */
// Hay otro update en la definicin de FLASHSPEC

#define HOLD_TIME	5
#define BEEP_TIME	100

#define NUM_ENTRIES	60
#define NUM_RECORDS	256

#define DINBUFSIZE 15
#define DOUTBUFSIZE 15

/* TCP/IP stuff */

#define MY_IP_ADDRESS   "192.168.1.52"
#define MY_NETMASK      "255.255.255.0"

#define MY_GATEWAY		"192.168.1.1" 		// only needed if out of local LAN

#define FTP_MAX_FNLEN 32
#memmap xmem
#use "dcrtcp.lib"
#use "http.lib"
#use "smtp.lib"
#use "ftp_client.lib"


/* use flash import facility */

#ximport "index.shtml"    index_html
#ximport "list.shtml"     list_html
#ximport "learn.shtml"    learn_html
#ximport "admin.shtml"    admin_html
#ximport "admail.shtml"   admail_html
#ximport "admftp.shtml"   admftp_html
#ximport "purgeid.shtml"  purgeid_html
#ximport "purgedb.shtml"  purgedb_html
#ximport "setdate.shtml"  setdate_html
#ximport "error.html"     error_html
#ximport "noroom.html"    noroom_html
#ximport "ok.html"	     ok_html
#ximport "ok2.html"	     ok2_html
#ximport "ok3.html"	     ok3_html
#ximport "rabbit1.gif"    rabbit1_gif

typedef struct {
	char name[21];
   unsigned char rfid[6];
} rfid_record;

typedef struct {
	int id;
   long time;
} time_record;


protected rfid_record rfid_base[NUM_ENTRIES];
protected time_record time_base[NUM_RECORDS];

protected int entries,records;

protected char from[21],to[21],subject[21],smtp_server[21],mail_time[9];
protected char user[21],password[21],filename[21],ftp_server[21],ftp_time[9];


const static char *months[]={
"ENE","FEB","MAR","ABR","MAY","JUN","JUL","AGO","SEP","OCT","NOV","DIC"
};

/* the default for / must be first */
const static HttpType http_types[] =
{
   { ".shtml", "text/html", shtml_handler}, // ssi
   { ".html", "text/html", NULL},           // html
   { ".cgi", "", NULL},                     // cgi
   { ".gif", "image/gif", NULL}
};

char date[12],time[9];			// store current date&time

/* Program states */
#define ESCRACHATE	1
#define LEARN			2
static int sstate;

int learn(HttpState*);
int list(HttpState*);
int show(HttpState*);
int setdate(HttpState*);
int setmail(HttpState*);
int setftp(HttpState*);
int purgeid(HttpState*);
int purgedb(HttpState*);
int delete(HttpState*);


// DC8.5 update
/* Hasta DC8.5, es posible definir slo un realm con nombre de
usuario y password, de modo de hacer una autenticacin simple.
Las versiones superiores tienen opciones ms potentes, cuyo
manejo y definicin de usuarios escapa al alcance de esta nota;
LO CUAL SIGNIFICA QUE ESTE PROGRAMA FUNCIONAR SIN AUTENTICACIN
EN VERSIONES DE DYNAMIC C SUPERIORES a 8.30 (con la cual se prob) */

#if CC_VER <= 0x0830
const static HttpRealm myrealm[]={"escra","chator","Escrachator"};
const static HttpSpec http_flashspec[] =
{
   { HTTPSPEC_FILE,  "/",              index_html,    NULL, 0, NULL, NULL},
   { HTTPSPEC_FILE,  "/index.shtml",   index_html,    NULL, 0, NULL, NULL},
   { HTTPSPEC_FILE,  "/learn.shtml",  	learn_html,	   NULL, 0, NULL, myrealm},
   { HTTPSPEC_FILE,  "/admin.shtml", 	admin_html,	   NULL, 0, NULL, myrealm},
   { HTTPSPEC_FILE,  "/admail.shtml", 	admail_html,   NULL, 0, NULL, myrealm},
   { HTTPSPEC_FILE,  "/admftp.shtml", 	admftp_html,   NULL, 0, NULL, myrealm},
   { HTTPSPEC_FILE,  "/purgeid.shtml", purgeid_html,  NULL, 0, NULL, myrealm},
   { HTTPSPEC_FILE,  "/purgedb.shtml", purgedb_html,  NULL, 0, NULL, myrealm},
   { HTTPSPEC_FILE,  "/list.shtml",  	list_html,	   NULL, 0, NULL, myrealm},
   { HTTPSPEC_FILE,  "/setdate.shtml",	setdate_html,  NULL, 0, NULL, myrealm},
   { HTTPSPEC_FILE,  "/ok.html",   		ok_html,	 	   NULL, 0, NULL, NULL},
   { HTTPSPEC_FILE,  "/ok2.html",   	ok2_html,  	   NULL, 0, NULL, NULL},
   { HTTPSPEC_FILE,  "/ok3.html",   	ok3_html,  	   NULL, 0, NULL, NULL},
   { HTTPSPEC_FILE,  "/error.html",   	error_html,	   NULL, 0, NULL, NULL},
   { HTTPSPEC_FILE,  "/noroom.html",   noroom_html,   NULL, 0, NULL, NULL},
   { HTTPSPEC_FILE,  "/rabbit1.gif",   rabbit1_gif,   NULL, 0, NULL, NULL},

   { HTTPSPEC_VARIABLE, "date", 0,    date, 		PTR16,   "%s", NULL},
   { HTTPSPEC_VARIABLE, "time", 0,    time, 		PTR16,   "%s", NULL},
   { HTTPSPEC_VARIABLE, "records", 0, &records, INT16,	"%d", NULL},
   { HTTPSPEC_VARIABLE, "entries", 0, &entries, INT16,	"%d", NULL},
   { HTTPSPEC_VARIABLE, "server",  0, smtp_server,PTR16, "%s", NULL},
   { HTTPSPEC_VARIABLE, "from", 	  0, from,		PTR16,   "%s", NULL},
   { HTTPSPEC_VARIABLE, "to", 	  0, to,			PTR16,   "%s", NULL},
   { HTTPSPEC_VARIABLE, "subject", 0, subject,	PTR16,   "%s", NULL},
   { HTTPSPEC_VARIABLE, "rtime",   0, mail_time,PTR16,   "%s", NULL},
   { HTTPSPEC_VARIABLE, "fserver", 0, ftp_server,PTR16,  "%s", NULL},
   { HTTPSPEC_VARIABLE, "user", 	  0, user,		PTR16,   "%s", NULL},
   { HTTPSPEC_VARIABLE, "password",0, password,	PTR16,   "%s", NULL},
   { HTTPSPEC_VARIABLE, "filename",0, filename,	PTR16,   "%s", NULL},
   { HTTPSPEC_VARIABLE, "ftime",   0, ftp_time, PTR16,   "%s", NULL},

   { HTTPSPEC_FUNCTION,	"/learn.cgi",		 0,  learn, 	0, NULL, myrealm},
   { HTTPSPEC_FUNCTION,	"/purgeid.cgi",  	 0,  purgeid, 	0, NULL, myrealm},
   { HTTPSPEC_FUNCTION,	"/purgedb.cgi",  	 0,  purgedb, 	0, NULL, myrealm},
   { HTTPSPEC_FUNCTION,	"/delete.cgi",		 0,  delete, 	0, NULL, myrealm},
   { HTTPSPEC_FUNCTION,	"/setdate.cgi",	 0,  setdate, 	0, NULL, myrealm},
   { HTTPSPEC_FUNCTION,	"/setmail.cgi",	 0,  setmail, 	0, NULL, myrealm},
   { HTTPSPEC_FUNCTION,	"/setftp.cgi",		 0,  setftp, 	0, NULL, myrealm},
   { HTTPSPEC_FUNCTION,	"show",		 		 0,  show, 		0, NULL, NULL},
   { HTTPSPEC_FUNCTION,	"list",		 		 0,  list, 		0, NULL, myrealm}
};
#else
const static HttpSpec http_flashspec[] =
{
   { HTTPSPEC_FILE,  "/",              index_html,    NULL, 0, NULL, NULL},
   { HTTPSPEC_FILE,  "/index.shtml",   index_html,    NULL, 0, NULL, NULL},
   { HTTPSPEC_FILE,  "/learn.shtml",  	learn_html,	   NULL, 0, NULL, NULL},
   { HTTPSPEC_FILE,  "/admin.shtml", 	admin_html,	   NULL, 0, NULL, NULL},
   { HTTPSPEC_FILE,  "/admail.shtml", 	admail_html,   NULL, 0, NULL, NULL},
   { HTTPSPEC_FILE,  "/admftp.shtml", 	admftp_html,   NULL, 0, NULL, NULL},
   { HTTPSPEC_FILE,  "/purgeid.shtml", purgeid_html,  NULL, 0, NULL, NULL},
   { HTTPSPEC_FILE,  "/purgedb.shtml", purgedb_html,  NULL, 0, NULL, NULL},
   { HTTPSPEC_FILE,  "/list.shtml",  	list_html,	   NULL, 0, NULL, NULL},
   { HTTPSPEC_FILE,  "/setdate.shtml",	setdate_html,  NULL, 0, NULL, NULL},
   { HTTPSPEC_FILE,  "/ok.html",   		ok_html,	 	   NULL, 0, NULL, NULL},
   { HTTPSPEC_FILE,  "/ok2.html",   	ok2_html,  	   NULL, 0, NULL, NULL},
   { HTTPSPEC_FILE,  "/ok3.html",   	ok3_html,  	   NULL, 0, NULL, NULL},
   { HTTPSPEC_FILE,  "/error.html",   	error_html,	   NULL, 0, NULL, NULL},
   { HTTPSPEC_FILE,  "/noroom.html",   noroom_html,   NULL, 0, NULL, NULL},
   { HTTPSPEC_FILE,  "/rabbit1.gif",   rabbit1_gif,   NULL, 0, NULL, NULL},

   { HTTPSPEC_VARIABLE, "date", 0,    date, 		PTR16,   "%s", NULL},
   { HTTPSPEC_VARIABLE, "time", 0,    time, 		PTR16,   "%s", NULL},
   { HTTPSPEC_VARIABLE, "records", 0, &records, INT16,	"%d", NULL},
   { HTTPSPEC_VARIABLE, "entries", 0, &entries, INT16,	"%d", NULL},
   { HTTPSPEC_VARIABLE, "server",  0, smtp_server,PTR16, "%s", NULL},
   { HTTPSPEC_VARIABLE, "from", 	  0, from,		PTR16,   "%s", NULL},
   { HTTPSPEC_VARIABLE, "to", 	  0, to,			PTR16,   "%s", NULL},
   { HTTPSPEC_VARIABLE, "subject", 0, subject,	PTR16,   "%s", NULL},
   { HTTPSPEC_VARIABLE, "rtime",   0, mail_time,PTR16,   "%s", NULL},
   { HTTPSPEC_VARIABLE, "fserver", 0, ftp_server,PTR16,  "%s", NULL},
   { HTTPSPEC_VARIABLE, "user", 	  0, user,		PTR16,   "%s", NULL},
   { HTTPSPEC_VARIABLE, "password",0, password,	PTR16,   "%s", NULL},
   { HTTPSPEC_VARIABLE, "filename",0, filename,	PTR16,   "%s", NULL},
   { HTTPSPEC_VARIABLE, "ftime",   0, ftp_time, PTR16,   "%s", NULL},

   { HTTPSPEC_FUNCTION,	"/learn.cgi",		 0,  learn, 	0, NULL, NULL},
   { HTTPSPEC_FUNCTION,	"/purgeid.cgi",  	 0,  purgeid, 	0, NULL, NULL},
   { HTTPSPEC_FUNCTION,	"/purgedb.cgi",  	 0,  purgedb, 	0, NULL, NULL},
   { HTTPSPEC_FUNCTION,	"/delete.cgi",		 0,  delete, 	0, NULL, NULL},
   { HTTPSPEC_FUNCTION,	"/setdate.cgi",	 0,  setdate, 	0, NULL, NULL},
   { HTTPSPEC_FUNCTION,	"/setmail.cgi",	 0,  setmail, 	0, NULL, NULL},
   { HTTPSPEC_FUNCTION,	"/setftp.cgi",		 0,  setftp, 	0, NULL, NULL},
   { HTTPSPEC_FUNCTION,	"show",		 		 0,  show, 		0, NULL, NULL},
   { HTTPSPEC_FUNCTION,	"list",		 		 0,  list, 		0, NULL, NULL}
};
#endif
// End DC8.5 update


#define REDIRECTHOST		MY_IP_ADDRESS
#define REDIRECTTOK     "http://" REDIRECTHOST "/ok.html"
#define REDIRECTTOK2    "http://" REDIRECTHOST "/ok2.html"
#define REDIRECTTOK3    "http://" REDIRECTHOST "/ok3.html"
#define REDIRECTTOERR   "http://" REDIRECTHOST "/error.html"
#define REDIRECTTOERR2  "http://" REDIRECTHOST "/noroom.html"

/* LCD control signals */
#define	LCD_E		5
#define	LCD_RS	0
#define	LCD_RW	1

/* LCD status bits */
#define BUSY	7



/* Low level functions */

void LCD_Write(int data)
{
	WrPortI ( PADR, &PADRShadow, data );		// write data
	WrPortI ( SPCR, &SPCRShadow, 0x84 );		// PA0-7 = Outputs, put data in bus
	BitWrPortI ( PEDR, &PEDRShadow, 0,LCD_RW );	// Lower RW (Write)
	BitWrPortI ( PEDR, &PEDRShadow, 1,LCD_E );	// Rise E
	BitWrPortI ( PEDR, &PEDRShadow, 0,LCD_E );	// Lower E
}

int LCD_Read()
{
int data;
	BitWrPortI ( PEDR, &PEDRShadow, 1,LCD_RW );	// Rise RW (Read)
	WrPortI ( SPCR, &SPCRShadow, 0x80 );		// PA0-7 = Inputs, remove data from bus
	BitWrPortI ( PEDR, &PEDRShadow, 1,LCD_E ); // Rise E
	data=RdPortI ( PADR );		   // read data
	BitWrPortI ( PEDR, &PEDRShadow, 0,LCD_E ); // Lower E
	return(data);
}

int LCD_Status()
{
int status;
	BitWrPortI ( PEDR, &PEDRShadow, 0,LCD_RS );	// Lower RS (Cmd)
	do {
		status=LCD_Read();		   // read status (busy flag+address)
	} while(status&(1<<BUSY));		// loop while busy=1
	return(status);
}

int LCD_ReadData()
{
	BitWrPortI ( PEDR, &PEDRShadow, 1,LCD_RS );	// Rise RS (Data)
	return(LCD_Read());
}

void LCD_WriteCmd(int cmd)
{
	LCD_Status();		// RS returns low
	LCD_Write(cmd);
}

void LCD_WriteData(int data)
{
	LCD_Status();
	BitWrPortI ( PEDR, &PEDRShadow, 1,LCD_RS );	// Rise RS (Data)
	LCD_Write(data);
}

void LCD_init ()
{
	WrPortI ( PEDR,&PEDRShadow,0x00 );		// Make sure Outputs are Low
	WrPortI ( PEDDR,&PEDDRShadow,'\B10100011' );	// PE0,1,5,7 = output
	WrPortI ( PEFR, &PEFRShadow, 0 );		// PE: no I/O strobe

	LCD_WriteCmd ( '\B00111000' );			// 8 bit mode, 2 (or more) lines, 5x7
	LCD_WriteCmd ( '\B00000110' );			// incr address and shift cursor with each character
	LCD_WriteCmd ( '\B00001100' );			// turn on display, no cursor, non-blinking
}

/* High level functions */

void LCD_show ( char *ptr )
{
	while (*ptr)
		LCD_WriteData (*ptr++);
}

void LCD_at (unsigned int row, unsigned int col)
{
	const static char acPos[4] = { 0x80,0xC0,0x94,0xD4 };

		LCD_WriteCmd( acPos[row]+col );
}


void LCD_clear ( void )
{
	LCD_at(0,0);		//address 0
	LCD_WriteCmd (0x10);			// home
	LCD_WriteCmd (0x01);			// clear the display
}

LCD_defchar(int charadd, int *pattern)
{
int i;
	LCD_WriteCmd((charadd<<3)^0x40); // CGRAM address
	for(i=0;i<8;i++)
		LCD_WriteData(pattern[i]);
}


/* RTC functions */

void get_datetime()
{
auto struct tm thetm;

	mktm(&thetm, SEC_TIMER);
	sprintf(date,"%02d %s %04d",thetm.tm_mday,months[thetm.tm_mon-1],1900+thetm.tm_year);
	sprintf(time,"%02d:%02d:%02d",thetm.tm_hour, thetm.tm_min, thetm.tm_sec);
}

void gen_filename(char *dest,char *source)
{
auto struct tm thetm;

	mktm(&thetm, SEC_TIMER);
	sprintf(dest,"%s_%02d%02d%02d.csv",source,thetm.tm_mday,thetm.tm_mon,thetm.tm_year-100);
}


#define MAX_FORMSIZE	21
typedef struct {
	char *name;
	char value[MAX_FORMSIZE];
} FORMType;
FORMType FORMSpec[5];

/*
 * Parse one token 'foo=bar', matching 'foo' to the name field in
 * the struct, and storing 'bar' into the value
 */
void parse_token(HttpState* state)
{
	auto int i, len;

	for(i=0; i<HTTP_MAXBUFFER; i++) {
		if(state->buffer[i] == '=')
			state->buffer[i] = '\0';
	}
	state->p = state->buffer + strlen(state->buffer) + 1;

	for(i=0; i<(sizeof(FORMSpec)/sizeof(FORMType)); i++) {
		if(!strcmp(FORMSpec[i].name,state->buffer)) {
			len = (strlen(state->p)>MAX_FORMSIZE) ? MAX_FORMSIZE - 1: strlen(state->p);
			strncpy(FORMSpec[i].value,state->p,1+len);
			FORMSpec[i].value[MAX_FORMSIZE - 1] = '\0';
		}
	}
}

/*
 * parse the url-encoded POST data into the FORMSpec struct
 * (ie: parse 'foo=bar&baz=qux' into the struct
 */
int parse_post(HttpState* state)
{
	auto int retval;

	while(1) {
		retval = sock_fastread(&state->s, state->p, 1);
		if(0 == retval) {
			*state->p = '\0';
			parse_token(state);

			return 1;
		}

		/* should this only be '&'? (allow the eoln as valid text?) */
		if((*state->p == '&') || (*state->p == '\r') || (*state->p == '\n')) {
			/* found one token */
			*state->p = '\0';
			parse_token(state);

			state->p = state->buffer;
		} else {
			state->p++;
		}

		if((state->p - state->buffer) > HTTP_MAXBUFFER) {
			/* input too long */
			return 1;
		}
	}

	return 0; /* end of data - loop again to give it time to write more */
}


int learn(HttpState* state)
{
char buffer[MAX_FORMSIZE];

	if(entries<NUM_ENTRIES){
		FORMSpec[0].name = "nombre";
		FORMSpec[0].value[0] = 0;									// init form data
		if(parse_post(state)) {
			if(*(http_urldecode(buffer,FORMSpec[0].value,MAX_FORMSIZE))) { // decode data, ignore empty strings
         	strcpy(FORMSpec[0].value,buffer);
				sstate=LEARN;
				cgi_redirectto(state,REDIRECTTOK);					// Show learn screen
			}
			else cgi_redirectto(state,REDIRECTTOERR);					// Show error screen
		} else {
			cgi_redirectto(state,REDIRECTTOERR);					// Show error screen
									// insert error handling here
		}
   }
   else	cgi_redirectto(state,REDIRECTTOERR2);					// Show error screen
	return(0);
}

int list(HttpState* state)
{
	auto int printline;

	if(state->substate >= entries)
		return 1;

	printline = state->substate;

	sprintf(state->buffer, "<tr><td>%2d<td>%s\r\n",printline,rfid_base[printline].name);
	state->headerlen = strlen(state->buffer);
	state->headeroff = 0;
	state->substate++;
	return(0);
}

int delete(HttpState* state)
{
char buffer[MAX_FORMSIZE];
int id,err,i;

		err=0;
		FORMSpec[0].name = "id";
		FORMSpec[0].value[0] = 0;									// init form data
		if(parse_post(state)) {
			if(*(http_urldecode(buffer,FORMSpec[0].value,MAX_FORMSIZE))) { // decode data, ignore empty strings
         	id=atoi(buffer);
            if((id>=0)&&(id<entries)&&entries){
            	for(i=id+1;i<entries;i++){
						strcpy(rfid_base[i-1].name,rfid_base[i].name);
						memcpy(rfid_base[i-1].rfid,rfid_base[i].rfid,5);
					}
               entries--;
					cgi_redirectto(state,REDIRECTTOK3);					// Show OK screen
            }
				else err=1;
			}
			else err=1;
		}
      else err=1;
      if(err)
			cgi_redirectto(state,REDIRECTTOERR);					// Show error screen
		return(0);
}

int show(HttpState* state)
{
	auto int printline;
	auto struct tm thetm;

	if(state->substate >= records)
		return 1;

	printline = records-(state->substate)-1;		// show backwards
	mktm(&thetm, time_base[printline].time);
	sprintf(state->buffer, "<tr><td>%02d %s %04d<td>%02d:%02d:%02d<td>%s\r\n",thetm.tm_mday,months[thetm.tm_mon-1],1900+thetm.tm_year,
			thetm.tm_hour, thetm.tm_min, thetm.tm_sec, rfid_base[time_base[printline].id].name);

	state->headerlen = strlen(state->buffer);
	state->headeroff = 0;
	state->substate++;
	return 0;
}

int purgeid(HttpState* state)
{
	entries=0;
	cgi_redirectto(state,REDIRECTTOK3);					// Show OK screen
	return 0;
}

int purgedb(HttpState* state)
{
	records=0;
	cgi_redirectto(state,REDIRECTTOK3);					// Show OK screen
	return 0;
}


int setmail(HttpState* state)
{
char buffer[MAX_FORMSIZE];
unsigned int err;

		FORMSpec[0].name = "server";
		FORMSpec[1].name = "from";
		FORMSpec[2].name = "to";
		FORMSpec[3].name = "subject";
		FORMSpec[4].name = "time";
		FORMSpec[0].value[0] = 0;									// init form data
		FORMSpec[1].value[0] = 0;									// init form data
		FORMSpec[2].value[0] = 0;									// init form data
		FORMSpec[3].value[0] = 0;									// init form data
		FORMSpec[4].value[0] = 0;									// init form data
      err=0;
		if(parse_post(state)) {
			if(*(http_urldecode(buffer,FORMSpec[0].value,MAX_FORMSIZE))) { // decode data, ignore empty strings
         	strcpy(smtp_server,buffer);
				if(*(http_urldecode(buffer,FORMSpec[1].value,MAX_FORMSIZE))) { // decode data, ignore empty strings
	         	strcpy(from,buffer);
					if(*(http_urldecode(buffer,FORMSpec[2].value,MAX_FORMSIZE))) { // decode data, ignore empty strings
		         	strcpy(to,buffer);
						if(*(http_urldecode(buffer,FORMSpec[3].value,MAX_FORMSIZE))) { // decode data, ignore empty strings
			         	strcpy(subject,buffer);
							if(*(http_urldecode(buffer,FORMSpec[4].value,MAX_FORMSIZE))) { // decode data, ignore empty strings
				         	strcpy(mail_time,buffer);
								cgi_redirectto(state,REDIRECTTOK3);					// Show OK screen
							}
							else err=1;
						}
						else err=1;
					}
					else err=1;
				}
				else err=1;
			}
			else err=1;
		}
      else err=1;
      if(err)
			cgi_redirectto(state,REDIRECTTOERR);					// Show error screen

		return(0);
}

int setftp(HttpState* state)
{
char buffer[MAX_FORMSIZE];
unsigned int err;

		FORMSpec[0].name = "server";
		FORMSpec[1].name = "user";
		FORMSpec[2].name = "password";
		FORMSpec[3].name = "filename";
		FORMSpec[4].name = "time";
		FORMSpec[0].value[0] = 0;									// init form data
		FORMSpec[1].value[0] = 0;									// init form data
		FORMSpec[2].value[0] = 0;									// init form data
		FORMSpec[3].value[0] = 0;									// init form data
		FORMSpec[4].value[0] = 0;									// init form data
      err=0;
		if(parse_post(state)) {
			if(*(http_urldecode(buffer,FORMSpec[0].value,MAX_FORMSIZE))) { // decode data, ignore empty strings
         	strcpy(ftp_server,buffer);
				if(*(http_urldecode(buffer,FORMSpec[1].value,MAX_FORMSIZE))) { // decode data, ignore empty strings
	         	strcpy(user,buffer);
					if(*(http_urldecode(buffer,FORMSpec[2].value,MAX_FORMSIZE))) { // decode data, ignore empty strings
		         	strcpy(password,buffer);
						if(*(http_urldecode(buffer,FORMSpec[3].value,MAX_FORMSIZE))) { // decode data, ignore empty strings
			         	strcpy(filename,buffer);
							if(*(http_urldecode(buffer,FORMSpec[4].value,MAX_FORMSIZE))) { // decode data, ignore empty strings
				         	strcpy(ftp_time,buffer);
								cgi_redirectto(state,REDIRECTTOK3);					// Show OK screen
							}
							else err=1;
						}
						else err=1;
					}
					else err=1;
				}
				else err=1;
			}
			else err=1;
		}
      else err=1;
      if(err)
			cgi_redirectto(state,REDIRECTTOERR);					// Show error screen

		return(0);
}


int setdate(HttpState* state)
{
char buffer[MAX_FORMSIZE],*p,*endptr;
unsigned int month, day, year, hour, minute, done;
struct tm rtc;					// time struct


		FORMSpec[0].name = "date";
		FORMSpec[0].value[0] = 0;									// init form data
		if(parse_post(state)) {
			if(*(http_urldecode(buffer,FORMSpec[0].value,MAX_FORMSIZE))) { // decode data, ignore empty strings
         	p=buffer;
            done=0;
				day = (unsigned int)strtod(p, &endptr);
				if (endptr != p && day > 0 && day < 32) {
					p = endptr + 1;
					month = (unsigned int)strtod(p, &endptr);
					if (endptr != p && month > 0 && month < 32) {
						p = endptr + 1;
						year = (unsigned int)strtod(p, &endptr);
						if (endptr != p && (year < 48 || year > 79)) {
							if (year < 48)
                     	year += 100;
							p = endptr + 1;
							hour = (unsigned int)strtod(p, &endptr);
							if (endptr != p && hour < 24) {
								p = endptr + 1;
								minute = (unsigned int)strtod(p, &endptr);
								if (minute < 60) {
										rtc.tm_sec  = 0;	   		// 0-59
										rtc.tm_min  = minute;		// 0-59
										rtc.tm_hour = hour;			// 0-23
										rtc.tm_mday = day;			// 1-31
										rtc.tm_mon  = month;			// 1-12
										rtc.tm_year = year;			// 80-147, add 1900 to get year
																	//		(i.e. 99 -> 1999)
										tm_wr(&rtc);					// set clock
                              done=1;
										cgi_redirectto(state,REDIRECTTOK2);					// Show learn screen
								}
							}
						}
					}
				}

				if (!done) {
					cgi_redirectto(state,REDIRECTTOERR);					// Show error screen
				}
			}
			else cgi_redirectto(state,REDIRECTTOERR);					// Show error screen
		} else {
			cgi_redirectto(state,REDIRECTTOERR);					// Show error screen
									// insert error handling here
		}
		return(0);
}

void decode(unsigned char *rfid)
{
int i;
	for(i=0;i<5;i++){
   	rfid[2*i+1]-=0x30;
      if(rfid[2*i+1]>=10)
       rfid[2*i+1]-=7;
   	rfid[2*i+2]-=0x30;
      if(rfid[2*i+2]>=10)
       rfid[2*i+2]-=7;
   	rfid[i]=(rfid[2*i+1]<<4) | rfid[2*i+2];
   }
}



int lookup(unsigned char *id)
{
int i;
	for(i=0;i<entries;i++){
   	if(!memcmp(id,rfid_base[i].rfid,5))
      	return(i);
   }
   return(-1);
}

int mail_generator(char * buf, int len, longword offset,
                                int flags, void * dhnd_data)
{
auto int *printline;
auto struct tm thetm;

	if (flags == SMTPDH_OUT) {
		printline = (int *)dhnd_data;	// Access our opaque data
		if((*printline) >= records)
			return 0;
		mktm(&thetm, time_base[(*printline)].time);
		sprintf(buf, "%02d %s %04d\t%02d:%02d:%02d\t%s\t\t%d\t%d\t%d\r\n",thetm.tm_mday,months[thetm.tm_mon-1],1900+thetm.tm_year,
			thetm.tm_hour, thetm.tm_min, thetm.tm_sec, rfid_base[time_base[(*printline)].id].name,
         thetm.tm_mday,thetm.tm_mon,1900+thetm.tm_year);
		(*printline)++;	// Increment line number (data is not opaque to us!).
		return(strlen(buf));
	}
	return -1;	// Indicate don't understand or care about this flag
}

int ftp_datahandler(char * data, int len, longword offset,
                   int flags, void * dhnd_data)
{
static printline;
auto struct tm thetm;

#GLOBAL_INIT {
	printline=0;
}

	switch (flags) {
		case FTPDH_IN:
			return 0;
		case FTPDH_OUT:
			if(printline >= records)
				return 0;
			mktm(&thetm, time_base[printline].time);
			sprintf(data, "%02d %s %04d,%02d:%02d:%02d,%s,%d,%d,%d\r\n",thetm.tm_mday,months[thetm.tm_mon-1],1900+thetm.tm_year,
				thetm.tm_hour, thetm.tm_min, thetm.tm_sec, rfid_base[time_base[printline].id].name,
         	thetm.tm_mday,thetm.tm_mon,1900+thetm.tm_year);
			printline++;	// Increment record number
			return(strlen(data));

		case FTPDH_END:
		case FTPDH_ABORT:
      	printline=0;
			return 0;
	}
	return -1;
}


/* MAIN PROGRAM */


main()
{
int i,sending,linenum,beep,ftpret,fsending;
unsigned char RFID[15];
char my_filename[32];
const static char wrong[]="El RFID leido";
static unsigned long timer;


#GLOBAL_INIT {
	sstate=ESCRACHATE;
}

	_sysIsSoftReset();	// recover any protected variables
	sending=0;
	fsending=0;
   beep=0;
	LCD_init();
	LCD_clear();
	LCD_show("Time logger v1.0   READY");
	LCD_at(1,0);
	LCD_show("(C)2004 Cika Electronica");
   timer=SEC_TIMER;					// restart hold timer
   sock_init();
   http_init();
   serDopen(9600);
   while(1){
   	costate {
      	if(beep){
         	BitWrPortI ( PEDR, &PEDRShadow, 1,7 );	// Enable buzzer
            waitfor(DelayMs(BEEP_TIME));
         	BitWrPortI ( PEDR, &PEDRShadow, 0,7 );	// Disable buzzer
            beep=0;
         }
      }
   	costate{
	      http_handler();						// TCP/IP stuff
         if(sending)
				if (smtp_mailtick()!=SMTP_PENDING)
   	      	sending=0;
         if(fsending)
				if((ftpret=ftp_client_tick())!= FTPC_AGAIN) {
  	  	      	fsending=0;
	         	if(ftpret==FTPC_OK)
	               records=0;						// remove sent records
            }
		}
		costate {
			do {
  	      	yield;
         	waitfor(DelaySec(1));				// waits until 1 second have passed
				get_datetime();						// update date & time strings
         } while((SEC_TIMER-timer)<HOLD_TIME); // do not update screen if timer is set
     	  	LCD_clear();
			LCD_at(0,4);							// show date & time
			LCD_show("Cika Time Logger");
			LCD_at(1,1);
			LCD_show(date);
			LCD_at(1,15);
			LCD_show(time);
		}
		costate {	// Handle RFID record
	      wfd i=cof_serDread(RFID,14,100); 	// yields until it gets a record
         if(i==14){
         	if((RFID[0]==2)&&(RFID[13]==3)){
            	decode(RFID);
	            beep=1;
	      	  	LCD_clear();
            	switch (sstate){
               case ESCRACHATE:
               	if((i=lookup(RFID))!= -1){
							LCD_at(0,1);
							LCD_show(time);
							LCD_at(0,12);
							LCD_show(date);
							LCD_at(1,2);
							LCD_show(rfid_base[i].name);
							time_base[records].id=i;
							time_base[records].time=SEC_TIMER;
                     if(records<NUM_RECORDS)	// overwrite last record if no room
                     	records++;
                  }
                  else {
							LCD_at(0,5);
							LCD_show(wrong);
							LCD_at(1,2);
							LCD_show("no figura en la base");
                  }
               	break;
               case LEARN:
               	if((i=lookup(RFID))!= -1){
							LCD_at(0,5);
							LCD_show(wrong);
							LCD_at(1,2);
							LCD_show("ya figura en la base");
                  }
                  else {
							LCD_at(0,2);
							LCD_show("Ingresado a nombre de");
							LCD_at(1,2);
							strcpy(rfid_base[entries].name,FORMSpec[0].value);
							memcpy(rfid_base[entries].rfid,RFID,5);
							LCD_show(rfid_base[entries++].name);
                  }
						sstate=ESCRACHATE;
               	break;
               default:
               	break;
               }
			      timer=SEC_TIMER;					// restart hold timer
            }
         }
		}
		costate {
			waitfor(!strncmp(time,mail_time,5));					// waits until right time
			if((!sending)&&records){
         	linenum=0;
				smtp_setserver(smtp_server);							// handle dynamically
         	smtp_sendmail(to, from, subject, NULL);			// No fixed message
				smtp_data_handler(mail_generator, &linenum, 0);	// Set message generator function
				sending=1;
				waitfor(DelaySec(65));									// Delay to avoid repeated msgs
         }
		}
		costate {
			waitfor(!strncmp(time,ftp_time,5));					// waits until right time
			if((!fsending)&&records){
         	gen_filename(my_filename,filename);
				if(!ftp_client_setup(resolve(ftp_server),0,user,password,
					FTP_MODE_UPLOAD,my_filename,NULL,NULL,0)) {
					ftp_data_handler(ftp_datahandler, NULL, 0);
					fsending=1;
					waitfor(DelaySec(65));									// Delay to avoid repeated msgs
            }
         }
		}
	}
   serDclose();

}

